﻿using PolymorphicDeSerialization.Models;
using System.Text.Json;
using System.Text.Json.Serialization;

namespace PolymorphicDeSerialization.Converters.Json
{
    public class HI1ObjectConverterFactory : JsonConverter<HI1Object>
    {
        public override bool CanConvert(Type typeToConvert)
        {
            return typeof(HI1Object).IsAssignableFrom(typeToConvert);
        }

        public override HI1Object? Read(ref Utf8JsonReader reader, Type typeToConvert, JsonSerializerOptions options)
        {
            if (reader.TokenType != JsonTokenType.StartObject)
            {
                throw new JsonException();
            }

            reader.Read();
            if (reader.TokenType != JsonTokenType.PropertyName)
            {
                throw new JsonException();
            }

            string? propertyName = reader.GetString();
            if (propertyName != "TypeDiscriminator")
            {
                throw new JsonException();
            }

            reader.Read();
            if (reader.TokenType != JsonTokenType.Number)
            {
                throw new JsonException();
            }

            TypeDiscriminator typeDiscriminator = (TypeDiscriminator)reader.GetInt32();
            HI1Object hi1Object = typeDiscriminator switch
            {
                TypeDiscriminator.LITaskObject => new LITaskObject(),
                _ => throw new JsonException()
            };

            while (reader.Read())
            {
                if (reader.TokenType == JsonTokenType.EndObject)
                {
                    return hi1Object;
                }

                if (reader.TokenType == JsonTokenType.PropertyName)
                {
                    propertyName = reader.GetString();
                    reader.Read();
                    switch (propertyName)
                    {
                        case "Reference":
                            var reference = reader.GetString();
                            ((LITaskObject)hi1Object).Reference = reference;
                            break;
                        case "TargetIdentifier":
                            string? targetIdentifier = reader.GetString();
                            ((LITaskObject)hi1Object).TargetIdentifier = targetIdentifier;
                            break;
                        case "ObjectIdentifier":
                            var objectIdentifier = reader.GetString();
                            hi1Object.ObjectIdentifier = objectIdentifier;
                            break;
                        case "Generation":
                            var generation = reader.GetInt32();
                            hi1Object.Generation = generation;
                            break;
                    }
                }
            }

            throw new JsonException();
        }

        public override void Write(Utf8JsonWriter writer, HI1Object value, JsonSerializerOptions options)
        {
            writer.WriteStartObject();

            if (value is LITaskObject liTaskObject)
            {
                writer.WriteNumber("TypeDiscriminator", (int)TypeDiscriminator.LITaskObject);
                writer.WriteString("Reference", liTaskObject.Reference);
                writer.WriteString("TargetIdentifier", liTaskObject.TargetIdentifier);
            }

            writer.WriteString("ObjectIdentifier", value.ObjectIdentifier);
            writer.WriteNumber("Generation", value.Generation);

            writer.WriteEndObject();
        }
    }
}