﻿using PolymorphicDeSerialization.Converters.Json;
using PolymorphicDeSerialization.Models;
using System.Text.Json;

// Make an LI Task Object
var lit = new LITaskObject
{
    Reference = "1",
    TargetIdentifier = "someperson",
    ObjectIdentifier = Guid.NewGuid().ToString(),
    Generation = 0
};

// Scenario 1: Normal serialization of an explicitly defined object.
PrintHeader("Serialize an object explicitly defined as LITaskObject.");
var litString = JsonSerializer.Serialize(lit);
Console.WriteLine(litString);
PrintFooter();

// Scenario 2: Normal serialization of an object assigned to a base type object.
PrintHeader("Now Serialize the same LITaskObject, but do so after assigning it to an object of type HI1Object.");
var hi1Object = lit;
var hi1ObjectString = JsonSerializer.Serialize(hi1Object);
Console.WriteLine(hi1ObjectString);
PrintFooter();

// Scenario 3: Assign the derived type to a base type property of an object. This will result in loss of data when serialized.
PrintHeader("Now make an HI1Message. This has a property of type HI1Object as it can represent any HI1Object.", false);
PrintHeader("Only the HI1Object properties will be serialized. The fields in the derived type will be lost.");
var hi1Message = new HI1Message
{
    HI1Object = lit
};
var hi1MessageString = JsonSerializer.Serialize(hi1Message);
Console.WriteLine(hi1MessageString);
PrintFooter();

// Scenario 4: Create a custom serializer to handle derived types when serializing an object with properties of HI1Object and not the derived types.
PrintHeader("Now make a JsonSerializerFactory specific to HI1Object and its derived types.", false);
PrintHeader("Notice the presence of a TypeDiscriminator value. This is necessary to ensure we can deserialize later.");
var serializeOptions = new JsonSerializerOptions();
serializeOptions.Converters.Add(new HI1ObjectConverterFactory());
var serializerWithOptions = JsonSerializer.Serialize(hi1Message, serializeOptions);
Console.WriteLine(serializerWithOptions);
PrintFooter();

// Scenario 5: Assign the derived type to a version of HI1Message that has a property of type LITaskObject instead of HI1Object.
PrintHeader("Now make an HI1ObjectWithExplicitObjectTypes. This has a property of type LITaskObject.");
var hi1v2Message = new HI1ObjectWithExplicitObjectTypes
{
    LITaskObject = lit
};
var hi1v2MessageString = JsonSerializer.Serialize(hi1v2Message);
Console.WriteLine(hi1v2MessageString);
PrintFooter();

void PrintHeader(string headerText, bool addNewLine = true)
{
    Console.WriteLine($"{headerText}{(addNewLine ? Environment.NewLine : "")}");
}
void PrintFooter()
{
    Console.WriteLine("\r\n======================================================================\r\n");
}